@@ -1,4 +1,4 @@
-#ifdef HAVE_CONFIG_H
+#if HAVE_CONFIG_H
 # include <config.h>
 #endif
 #include <stdlib.h>
