@@ -1,6 +1,6 @@
 /* -*- mode: c; c-basic-offset: 2 -*- */
 /* gifwrite.c - Functions to write GIFs.
-   Copyright (C) 1997-2018 Eddie Kohler, ekohler@gmail.com
+   Copyright (C) 1997-2019 Eddie Kohler, ekohler@gmail.com
    This file is part of the LCDF GIF library.
 
    The LCDF GIF library is free software. It is distributed under the GNU
@@ -23,9 +23,9 @@
 extern "C" {
 #endif
 
-#define WRITE_BUFFER_SIZE	255
-#define NODES_SIZE		GIF_MAX_CODE
-#define LINKS_SIZE		GIF_MAX_CODE
+#define WRITE_BUFFER_SIZE       255
+#define NODES_SIZE              GIF_MAX_CODE
+#define LINKS_SIZE              GIF_MAX_CODE
 
 /* 1.Aug.1999 - Removed code hashing in favor of an adaptive tree strategy
    based on Whirlgif-3.04, written by Hans Dinsen-Hansen <dino@danbbs.dk>. Mr.
@@ -46,9 +46,9 @@ extern "C" {
    very small numbers of colors (2, 4, 8) won't get the speed benefits of
    TABLE nodes. */
 
-#define TABLE_TYPE		0
-#define LINKS_TYPE		1
-#define MAX_LINKS_TYPE		5
+#define TABLE_TYPE              0
+#define LINKS_TYPE              1
+#define MAX_LINKS_TYPE          5
 typedef struct Gif_Node {
   Gif_Code code;
   uint8_t type;
@@ -86,8 +86,8 @@ struct Gif_Writer {
 };
 
 
-#define gifputbyte(b, grr)	((*grr->byte_putter)(b, grr))
-#define gifputblock(b, l, grr)	((*grr->block_putter)(b, l, grr))
+#define gifputbyte(b, grr)      ((*grr->byte_putter)(b, grr))
+#define gifputblock(b, l, grr)  ((*grr->block_putter)(b, l, grr))
 
 static inline void
 gifputunsigned(uint16_t uns, Gif_Writer *grr)
@@ -204,6 +204,48 @@ gfc_lookup(Gif_CodeTable *gfc, Gif_Node *node, uint8_t suffix)
   }
 }
 
+/* Used to hold accumulated error for the current candidate match */
+typedef struct gfc_rgbdiff {signed short r, g, b;} gfc_rgbdiff;
+
+/* Difference (MSE) between given color indexes + dithering error */
+static inline unsigned int color_diff(Gif_Color a, Gif_Color b, int a_transparent, int b_transparent, gfc_rgbdiff dither)
+{
+  /* if one is transparent and the other is not, then return maximum difference */
+  /* TODO: figure out what color is in the canvas under the transparent pixel and match against that */
+  if (a_transparent != b_transparent) return 1<<25;
+
+  /* Two transparent colors are identical */
+  if (a_transparent) return 0;
+
+  /* squared error with or without dithering. */
+  unsigned int dith = (a.gfc_red-b.gfc_red+dither.r)*(a.gfc_red-b.gfc_red+dither.r)
+  + (a.gfc_green-b.gfc_green+dither.g)*(a.gfc_green-b.gfc_green+dither.g)
+  + (a.gfc_blue-b.gfc_blue+dither.b)*(a.gfc_blue-b.gfc_blue+dither.b);
+
+  unsigned int undith = (a.gfc_red-b.gfc_red+dither.r/2)*(a.gfc_red-b.gfc_red+dither.r/2)
+  + (a.gfc_green-b.gfc_green+dither.g/2)*(a.gfc_green-b.gfc_green+dither.g/2)
+  + (a.gfc_blue-b.gfc_blue+dither.b/2)*(a.gfc_blue-b.gfc_blue+dither.b/2);
+
+  /* Smaller error always wins, under assumption that dithering is not required and it's only done opportunistically */
+  return dith < undith ? dith : undith;
+}
+
+/* difference between expected color a+dither and color b (used to calculate dithering required) */
+static inline gfc_rgbdiff diffused_difference(Gif_Color a, Gif_Color b, int a_transparent, int b_transparent, gfc_rgbdiff dither)
+{
+  gfc_rgbdiff d;
+  if (a_transparent || b_transparent) {
+    d.r = d.g = d.b = 0;
+  } else {
+    d.r = a.gfc_red - b.gfc_red + dither.r * 3 / 4;
+    d.g = a.gfc_green - b.gfc_green + dither.g * 3 / 4;
+    d.b = a.gfc_blue - b.gfc_blue + dither.b * 3 / 4;
+  }
+  return d;
+}
+
+static inline uint8_t gif_pixel_at_pos(Gif_Image *gfi, unsigned pos);
+
 static void
 gfc_change_node_to_table(Gif_CodeTable *gfc, Gif_Node *work_node,
                          Gif_Node *next_node)
@@ -273,9 +315,97 @@ gif_line_endpos(Gif_Image *gfi, unsigned pos)
   return (y + 1) * gfi->width;
 }
 
+struct selected_node {
+  Gif_Node *node; /* which node has been chosen by gfc_lookup_lossy */
+  unsigned long pos, /* where the node ends */
+  diff; /* what is the overall quality loss for that node */
+};
+
+static inline void
+gfc_lookup_lossy_try_node(Gif_CodeTable *gfc, const Gif_Colormap *gfcm, Gif_Image *gfi,
+  unsigned pos, Gif_Node *node, uint8_t suffix, uint8_t next_suffix,
+  gfc_rgbdiff dither, unsigned long base_diff, const unsigned int max_diff, struct selected_node *best_t);
+
+/* Recursive loop
+ * Find node that is descendant of node (or start new search if work_node is null) that best matches pixels starting at pos
+ * base_diff and dither are distortion from search made so far */
+static struct selected_node
+gfc_lookup_lossy(Gif_CodeTable *gfc, const Gif_Colormap *gfcm, Gif_Image *gfi,
+  unsigned pos, Gif_Node *node, unsigned long base_diff, gfc_rgbdiff dither, const unsigned int max_diff)
+{
+  unsigned image_endpos = gfi->width * gfi->height;
+
+  struct selected_node best_t = {node, pos, base_diff};
+  if (pos >= image_endpos) return best_t;
+
+  uint8_t suffix = gif_pixel_at_pos(gfi, pos);
+  assert(!node || (node >= gfc->nodes && node < gfc->nodes + NODES_SIZE));
+  assert(suffix < gfc->clear_code);
+  if (!node) {
+    gfc_rgbdiff zero_diff = {0, 0, 0};
+    /* prefix of the new node must be same as suffix of previously added node */
+    return gfc_lookup_lossy(gfc, gfcm, gfi, pos+1, &gfc->nodes[suffix], base_diff, zero_diff, max_diff);
+  }
+
+  /* search all nodes that are less than max_diff different from the desired pixel */
+  if (node->type == TABLE_TYPE) {
+    int i;
+    for(i=0; i < gfc->clear_code; i++) {
+      if (!node->child.m[i]) continue;
+      gfc_lookup_lossy_try_node(gfc, gfcm, gfi, pos, node->child.m[i], suffix, i, dither, base_diff, max_diff, &best_t);
+    }
+  }
+  else {
+    for (node = node->child.s; node; node = node->sibling) {
+      gfc_lookup_lossy_try_node(gfc, gfcm, gfi, pos, node, suffix, node->suffix, dither, base_diff, max_diff, &best_t);
+    }
+  }
+
+  return best_t;
+}
+
+/**
+ * Replaces best_t with a new node if it's better
+ *
+ * @param node        Current node to search
+ * @param suffix      Previous pixel
+ * @param next_suffix Next pixel to evaluate (must correspond to the node given)
+ * @param dither      Desired dithering
+ * @param base_diff   Difference accumulated in the search so far
+ * @param max_diff    Maximum allowed pixel difference
+ * @param best_t      Current best candidate (input/output argument)
+ */
+static inline void
+gfc_lookup_lossy_try_node(Gif_CodeTable *gfc, const Gif_Colormap *gfcm, Gif_Image *gfi,
+  unsigned pos, Gif_Node *node, uint8_t suffix, uint8_t next_suffix,
+  gfc_rgbdiff dither, unsigned long base_diff, const unsigned int max_diff, struct selected_node *best_t)
+{
+  unsigned int diff = suffix == next_suffix ? 0 : color_diff(gfcm->col[suffix], gfcm->col[next_suffix], suffix == gfi->transparent, next_suffix == gfi->transparent, dither);
+  if (diff <= max_diff) {
+    gfc_rgbdiff new_dither = diffused_difference(gfcm->col[suffix], gfcm->col[next_suffix], suffix == gfi->transparent, next_suffix == gfi->transparent, dither);
+    /* if the candidate pixel is good enough, check all possible continuations of that dictionary string */
+    struct selected_node t = gfc_lookup_lossy(gfc, gfcm, gfi, pos+1, node, base_diff + diff, new_dither, max_diff);
+
+    /* search is biased towards finding longest candidate that is below treshold rather than a match with minimum average error */
+    if (t.pos > best_t->pos || (t.pos == best_t->pos && t.diff < best_t->diff)) {
+      *best_t = t;
+    }
+  }
+}
+
+static inline uint8_t
+gif_pixel_at_pos(Gif_Image *gfi, unsigned pos)
+{
+  unsigned y = pos / gfi->width, x = pos - y * gfi->width;
+  if (!gfi->interlace)
+    return gfi->img[y][x];
+  else
+    return gfi->img[Gif_InterlaceLine(y, gfi->height)][x];
+}
+
 static int
-write_compressed_data(Gif_Image *gfi,
-		      int min_code_bits, Gif_Writer *grr)
+write_compressed_data(Gif_Stream *gfs, Gif_Image *gfi,
+                      int min_code_bits, Gif_Writer *grr)
 {
   Gif_CodeTable* gfc = &grr->code_table;
   uint8_t stack_buffer[512 - 24];
@@ -286,6 +416,7 @@ write_compressed_data(Gif_Image *gfi,
   unsigned pos;
   unsigned clear_bufpos, clear_pos;
   unsigned line_endpos;
+  unsigned image_endpos;
   const uint8_t *imageline;
 
   unsigned run = 0;
@@ -298,6 +429,7 @@ write_compressed_data(Gif_Image *gfi,
   Gif_Code next_code = 0;
   Gif_Code output_code;
   uint8_t suffix;
+  Gif_Colormap *gfcm;
 
   int cur_code_bits;
 
@@ -312,14 +444,19 @@ write_compressed_data(Gif_Image *gfi,
   /* next_code set by first runthrough of output clear_code */
   GIF_DEBUG(("clear(%d) eoi(%d) bits(%d) ", CLEAR_CODE, EOI_CODE, cur_code_bits));
 
-  work_node = 0;
+  work_node = NULL;
   output_code = CLEAR_CODE;
   /* Because output_code is clear_code, we'll initialize next_code, et al.
      below. */
 
   pos = clear_pos = clear_bufpos = 0;
-  line_endpos = gfi->width;
-  imageline = gif_imageline(gfi, pos);
+  if (grr->gcinfo.loss) {
+    image_endpos = gfi->height * gfi->width;
+    gfcm = (gfi->local ? gfi->local : gfs->global);
+  } else {
+    line_endpos = gfi->width;
+    imageline = gif_imageline(gfi, pos);
+  }
 
   while (1) {
 
@@ -391,81 +528,143 @@ write_compressed_data(Gif_Image *gfi,
 
     /*****
      * Find the next code to output. */
+    if (grr->gcinfo.loss) {
+      gfc_rgbdiff zero_diff = {0, 0, 0};
+      struct selected_node t = gfc_lookup_lossy(gfc, gfcm, gfi, pos, NULL, 0, zero_diff, grr->gcinfo.loss * 10);
 
-    /* If height is 0 -- no more pixels to write -- we output work_node next
-       time around. */
-    while (imageline) {
-      suffix = *imageline;
-      next_node = gfc_lookup(gfc, work_node, suffix);
+      work_node = t.node;
+      run = t.pos - pos;
+      pos = t.pos;
 
-      imageline++;
-      pos++;
-      if (pos == line_endpos) {
-	imageline = gif_imageline(gfi, pos);
-        line_endpos += gfi->width;
-      }
+      if (pos < image_endpos) {
+        /* Output the current code. */
+        if (next_code < GIF_MAX_CODE) {
+          gfc_define(gfc, work_node, gif_pixel_at_pos(gfi, pos), next_code);
+          next_code++;
+        } else
+          next_code = GIF_MAX_CODE + 1; /* to match "> CUR_BUMP_CODE" above */
 
-      if (next_node) {
-        work_node = next_node;
-        ++run;
-        continue;
-      }
+        /* Check whether to clear table. */
+        if (next_code > 4094) {
+          int do_clear = grr->gcinfo.flags & GIF_WRITE_EAGER_CLEAR;
 
-      /* Output the current code. */
-      if (next_code < GIF_MAX_CODE) {
-        gfc_define(gfc, work_node, suffix, next_code);
-        next_code++;
-      } else
-        next_code = GIF_MAX_CODE + 1; /* to match "> CUR_BUMP_CODE" above */
+          if (!do_clear) {
+            unsigned pixels_left = image_endpos - pos - 1;
+            if (pixels_left) {
+              /* Always clear if run_ewma gets small relative to
+                 min_code_bits. Otherwise, clear if #images/run is smaller
+                 than an empirical threshold, meaning it will take more than
+                 3000 or so average runs to complete the image. */
+              if (run_ewma < ((36U << RUN_EWMA_SCALE) / min_code_bits)
+                  || pixels_left > UINT_MAX / RUN_INV_THRESH
+                  || run_ewma < pixels_left * RUN_INV_THRESH)
+                do_clear = 1;
+            }
+          }
 
-      /* Check whether to clear table. */
-      if (next_code > 4094) {
-        int do_clear = grr->gcinfo.flags & GIF_WRITE_EAGER_CLEAR;
+          if ((do_clear || run < 7) && !clear_pos) {
+            clear_pos = pos - run;
+            clear_bufpos = bufpos;
+          } else if (!do_clear && run > 50)
+            clear_pos = clear_bufpos = 0;
 
-        if (!do_clear) {
-          unsigned pixels_left = gfi->width * gfi->height - pos;
-          if (pixels_left) {
-            /* Always clear if run_ewma gets small relative to
-               min_code_bits. Otherwise, clear if #images/run is smaller
-               than an empirical threshold, meaning it will take more than
-               3000 or so average runs to complete the image. */
-            if (run_ewma < ((36U << RUN_EWMA_SCALE) / min_code_bits)
-                || pixels_left > UINT_MAX / RUN_INV_THRESH
-                || run_ewma < pixels_left * RUN_INV_THRESH)
-              do_clear = 1;
+          if (do_clear) {
+            GIF_DEBUG(("rewind %u pixels/%d bits", pos + 1 - clear_pos, bufpos + cur_code_bits - clear_bufpos));
+            output_code = CLEAR_CODE;
+            pos = clear_pos;
+
+            bufpos = clear_bufpos;
+            buf[bufpos >> 3] &= (1 << (bufpos & 7)) - 1;
+            grr->cleared = 1;
+            continue;
           }
         }
 
-        if ((do_clear || run < 7) && !clear_pos) {
-          clear_pos = pos - (run + 1);
-          clear_bufpos = bufpos;
-        } else if (!do_clear && run > 50)
-          clear_pos = clear_bufpos = 0;
-
-        if (do_clear) {
-          GIF_DEBUG(("rewind %u pixels/%d bits ", pos - clear_pos, bufpos + cur_code_bits - clear_bufpos));
-          output_code = CLEAR_CODE;
-          pos = clear_pos;
-          imageline = gif_imageline(gfi, pos);
-          line_endpos = gif_line_endpos(gfi, pos);
-          bufpos = clear_bufpos;
-          buf[bufpos >> 3] &= (1 << (bufpos & 7)) - 1;
-          work_node = 0;
-          grr->cleared = 1;
-          goto found_output_code;
-        }
+        /* Adjust current run length average. */
+        run = (run << RUN_EWMA_SCALE) + (1 << (RUN_EWMA_SHIFT - 1));
+        if (run < run_ewma)
+          run_ewma -= (run_ewma - run) >> RUN_EWMA_SHIFT;
+        else
+          run_ewma += (run - run_ewma) >> RUN_EWMA_SHIFT;
       }
 
-      output_code = work_node->code;
-      work_node = &gfc->nodes[suffix];
-      goto found_output_code;
+      output_code = (work_node ? work_node->code : EOI_CODE);
+    } else {
+      /* If height is 0 -- no more pixels to write -- we output work_node next
+         time around. */
+      while (imageline) {
+        suffix = *imageline;
+        next_node = gfc_lookup(gfc, work_node, suffix);
+
+        imageline++;
+        pos++;
+        if (pos == line_endpos) {
+          imageline = gif_imageline(gfi, pos);
+          line_endpos += gfi->width;
+        }
+
+        if (next_node) {
+          work_node = next_node;
+          ++run;
+          continue;
+        }
+
+        /* Output the current code. */
+        if (next_code < GIF_MAX_CODE) {
+          gfc_define(gfc, work_node, suffix, next_code);
+          next_code++;
+        } else
+          next_code = GIF_MAX_CODE + 1; /* to match "> CUR_BUMP_CODE" above */
+
+        /* Check whether to clear table. */
+        if (next_code > 4094) {
+          int do_clear = grr->gcinfo.flags & GIF_WRITE_EAGER_CLEAR;
+
+          if (!do_clear) {
+            unsigned pixels_left = gfi->width * gfi->height - pos;
+            if (pixels_left) {
+              /* Always clear if run_ewma gets small relative to
+                 min_code_bits. Otherwise, clear if #images/run is smaller
+                 than an empirical threshold, meaning it will take more than
+                 3000 or so average runs to complete the image. */
+              if (run_ewma < ((36U << RUN_EWMA_SCALE) / min_code_bits)
+                  || pixels_left > UINT_MAX / RUN_INV_THRESH
+                  || run_ewma < pixels_left * RUN_INV_THRESH)
+                do_clear = 1;
+            }
+          }
+
+          if ((do_clear || run < 7) && !clear_pos) {
+            clear_pos = pos - (run + 1);
+            clear_bufpos = bufpos;
+          } else if (!do_clear && run > 50)
+            clear_pos = clear_bufpos = 0;
+
+          if (do_clear) {
+            GIF_DEBUG(("rewind %u pixels/%d bits ", pos - clear_pos, bufpos + cur_code_bits - clear_bufpos));
+            output_code = CLEAR_CODE;
+            pos = clear_pos;
+            imageline = gif_imageline(gfi, pos);
+            line_endpos = gif_line_endpos(gfi, pos);
+            bufpos = clear_bufpos;
+            buf[bufpos >> 3] &= (1 << (bufpos & 7)) - 1;
+            work_node = NULL;
+            grr->cleared = 1;
+            goto found_output_code;
+          }
+        }
+
+        output_code = work_node->code;
+        work_node = &gfc->nodes[suffix];
+        goto found_output_code;
+      }
+
+      /* Ran out of data if we get here. */
+      output_code = (work_node ? work_node->code : EOI_CODE);
+      work_node = NULL;
+
+      found_output_code: ;
     }
-
-    /* Ran out of data if we get here. */
-    output_code = (work_node ? work_node->code : EOI_CODE);
-    work_node = 0;
-
-   found_output_code: ;
   }
 
   /* Output memory buffer to stream. */
@@ -504,8 +703,8 @@ calculate_min_code_bits(Gif_Image *gfi, const Gif_Writer *grr)
     for (y = 0; y < height && colors_used < 128; y++) {
       uint8_t *data = gfi->img[y];
       for (x = width; x > 0; x--, data++)
-	if (*data > colors_used)
-	  colors_used = *data;
+        if (*data > colors_used)
+          colors_used = *data;
     }
     colors_used++;
 
@@ -518,7 +717,7 @@ calculate_min_code_bits(Gif_Image *gfi, const Gif_Writer *grr)
     colors_used = 256;
   }
 
-  min_code_bits = 2;		/* min_code_bits of 1 isn't allowed */
+  min_code_bits = 2;            /* min_code_bits of 1 isn't allowed */
   i = 4;
   while (i < colors_used) {
     min_code_bits++;
@@ -530,7 +729,7 @@ calculate_min_code_bits(Gif_Image *gfi, const Gif_Writer *grr)
 
 
 static int get_color_table_size(const Gif_Stream *gfs, Gif_Image *gfi,
-				Gif_Writer *grr);
+                                Gif_Writer *grr);
 
 static void
 save_compression_result(Gif_Image *gfi, Gif_Writer *grr, int ok)
@@ -554,7 +753,7 @@ save_compression_result(Gif_Image *gfi, Gif_Writer *grr, int ok)
 
 int
 Gif_FullCompressImage(Gif_Stream *gfs, Gif_Image *gfi,
-		      const Gif_CompressInfo *gcinfo)
+                      const Gif_CompressInfo *gcinfo)
 {
   int ok = 0;
   uint8_t min_code_bits;
@@ -570,14 +769,14 @@ Gif_FullCompressImage(Gif_Stream *gfs, Gif_Image *gfi,
   grr.local_size = get_color_table_size(gfs, gfi, &grr);
 
   min_code_bits = calculate_min_code_bits(gfi, &grr);
-  ok = write_compressed_data(gfi, min_code_bits, &grr);
+  ok = write_compressed_data(gfs, gfi, min_code_bits, &grr);
   save_compression_result(gfi, &grr, ok);
 
   if ((grr.gcinfo.flags & (GIF_WRITE_OPTIMIZE | GIF_WRITE_EAGER_CLEAR))
       == GIF_WRITE_OPTIMIZE
       && grr.cleared && ok) {
     grr.gcinfo.flags |= GIF_WRITE_EAGER_CLEAR | GIF_WRITE_SHRINK;
-    if (write_compressed_data(gfi, min_code_bits, &grr))
+    if (write_compressed_data(gfs, gfi, min_code_bits, &grr))
       save_compression_result(gfi, &grr, 1);
   }
 
@@ -606,8 +805,8 @@ get_color_table_size(const Gif_Stream *gfs, Gif_Image *gfi, Gif_Writer *grr)
       ncol = gfi->transparent + 1;
     else if (!gfi)
       for (i = 0; i < gfs->nimages; i++)
-	if (gfs->images[i]->transparent >= ncol)
-	  ncol = gfs->images[i]->transparent + 1;
+        if (gfs->images[i]->transparent >= ncol)
+          ncol = gfs->images[i]->transparent + 1;
   }
 
   /* Make sure the colormap is a power of two entries! */
@@ -687,11 +886,11 @@ write_image(Gif_Stream *gfs, Gif_Image *gfi, Gif_Writer *grr)
 
   } else if (!gfi->img) {
     Gif_UncompressImage(gfs, gfi);
-    write_compressed_data(gfi, min_code_bits, grr);
+    write_compressed_data(gfs, gfi, min_code_bits, grr);
     Gif_ReleaseUncompressedImage(gfi);
 
   } else
-    write_compressed_data(gfi, min_code_bits, grr);
+    write_compressed_data(gfs, gfi, min_code_bits, grr);
 
   return 1;
 }
@@ -700,7 +899,7 @@ write_image(Gif_Stream *gfs, Gif_Image *gfi, Gif_Writer *grr)
 static void
 write_logical_screen_descriptor(Gif_Stream *gfs, Gif_Writer *grr)
 {
-  uint8_t packed = 0x70;		/* high resolution colors */
+  uint8_t packed = 0x70;                /* high resolution colors */
   grr->global_size = get_color_table_size(gfs, 0, grr);
 
   Gif_CalculateScreenSize(gfs, 0);
@@ -719,7 +918,7 @@ write_logical_screen_descriptor(Gif_Stream *gfs, Gif_Writer *grr)
     gifputbyte(gfs->background, grr);
   else
     gifputbyte(255, grr);
-  gifputbyte(0, grr);		/* no aspect ratio information */
+  gifputbyte(0, grr);           /* no aspect ratio information */
 
   if (grr->global_size > 0)
     write_color_table(gfs->global, grr->global_size, grr);
@@ -798,11 +997,11 @@ static void
 write_generic_extension(Gif_Extension *gfex, Gif_Writer *grr)
 {
   uint32_t pos = 0;
-  if (gfex->kind < 0) return;	/* ignore our private extensions */
+  if (gfex->kind < 0) return;   /* ignore our private extensions */
 
   gifputbyte('!', grr);
   gifputbyte(gfex->kind, grr);
-  if (gfex->kind == 255) {	/* an application extension */
+  if (gfex->kind == 255) {      /* an application extension */
     if (gfex->applength) {
       gifputbyte(gfex->applength, grr);
       gifputblock((const uint8_t*) gfex->appname, gfex->applength, grr);
@@ -872,7 +1071,7 @@ write_gif(Gif_Stream *gfs, Gif_Writer *grr)
 
 int
 Gif_FullWriteFile(Gif_Stream *gfs, const Gif_CompressInfo *gcinfo,
-		  FILE *f)
+                  FILE *f)
 {
   Gif_Writer grr;
   int ok = gif_writer_init(&grr, f, gcinfo)
