@@ -1,5 +1,5 @@
 /* quantize.c - Histograms and quantization for gifsicle.
-   Copyright (C) 1997-2018 Eddie Kohler, ekohler@gmail.com
+   Copyright (C) 1997-2019 Eddie Kohler, ekohler@gmail.com
    This file is part of gifsicle.
 
    Gifsicle is free software. It is distributed under the GNU Public License,
@@ -1620,11 +1620,6 @@ colormap_stream(Gif_Stream* gfs, Gif_Colormap* new_cm, Gt_OutputData* od)
       Gif_ReleaseCompressedImage(gfi);
       Gif_SetUncompressedImage(gfi, new_data, Gif_Free, 0);
 
-      if (only_compressed) {
-        Gif_FullCompressImage(gfs, gfi, &gif_write_info);
-        Gif_ReleaseUncompressedImage(gfi);
-      }
-
       /* update count of used colors */
       for (j = 0; j < 256; j++)
         new_col[j].pixel += histogram[j];
@@ -1643,6 +1638,12 @@ colormap_stream(Gif_Stream* gfs, Gif_Colormap* new_cm, Gt_OutputData* od)
       Gif_DeleteColormap(gfi->local);
       gfi->local = 0;
     }
+
+    /* 1.92: recompress *after* deleting the local colormap */
+    if (gfcm && only_compressed) {
+        Gif_FullCompressImage(gfs, gfi, &gif_write_info);
+        Gif_ReleaseUncompressedImage(gfi);
+    }
   }
 
   /* Set new_cm->ncol from new_ncol. We didn't update new_cm->ncol before so
